/*
 * sdl-color.c
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <linux/videodev.h>
#include <SDL/SDL.h>

#include "conversions.h"

#define BUFSZ   (1280*1024)

static unsigned char *buf;
static unsigned char *buf2;

/* Subtract the `struct timeval' values X and Y, storing the result in RESULT.
 * Return 1 if the difference is negative, otherwise 0.
 */
int timeval_subtract(struct timeval *result, struct timeval *x, struct timeval *y)
{
    /* Perform the carry for the later subtraction by updating Y.
     */
    if (x->tv_usec < y->tv_usec) {
        int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
        y->tv_usec -= 1000000 * nsec;
        y->tv_sec += nsec;
    }
    if (x->tv_usec - y->tv_usec > 1000000) {
        int nsec = (x->tv_usec - y->tv_usec) / 1000000;
        y->tv_usec += 1000000 * nsec;
        y->tv_sec -= nsec;
    }

    /* Compute the time remaining to wait. `tv_usec' is certainly positive.
     */
    result->tv_sec = x->tv_sec - y->tv_sec;
    result->tv_usec = x->tv_usec - y->tv_usec;

    /* Return 1 if result is negative.
     */
    return x->tv_sec < y->tv_sec;
}

int main(int argc, char *argv[])
{
    int fd;
    struct video_window vwin;
    unsigned frame_nr;
    struct timeval t1, t2, et;
    double total_sec;
    SDL_Surface *screen, *frame;
    SDL_Event event;
    int quit;
    int display = 1;

    if (argc != 3) {
        fprintf(stderr, "usage: %s WIDTH HEIGHT\n", argv[0]);
        exit(1);
    }

    fd = open("/dev/video", O_RDWR);
    if (fd < 0) {
        perror("open");
        exit(1);
    }

    if (ioctl(fd, VIDIOCGWIN, &vwin) < 0) {
        perror("ioctl(VIDIOCGWIN)");
        exit(1);
    }
    vwin.width = atoi(argv[1]);
    vwin.height = atoi(argv[2]);
    if (ioctl(fd, VIDIOCSWIN, &vwin) < 0) {
        perror("ioctl(VIDIOCSWIN)");
        exit(1);
    }

    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
        fprintf(stderr, "SDL_Init(): %s: FAILED\n", SDL_GetError());
        exit(1);
    }

    if (atexit(SDL_Quit)) {
        fprintf(stderr, "atexit(): FAILED\n");
        SDL_Quit();
        exit(1);
    }

    screen = SDL_SetVideoMode(vwin.width, vwin.height,24,SDL_HWSURFACE|SDL_DOUBLEBUF);
    if (!screen) {
        fprintf(stderr, "SDL_SetVideoMode(): %s: FAILED\n", SDL_GetError());
        exit(1);
    }

    buf=malloc(vwin.height*vwin.width);
    buf2=malloc(vwin.height*vwin.width*3);

    frame = SDL_CreateRGBSurfaceFrom(buf2, vwin.width, vwin.height, 24, vwin.width*3, 0xff, 0xff00, 0xff0000, 0);
    if (!frame) {
        fprintf(stderr, "SDL_CreateRGBSurfaceFrom(): %s: FAILED\n", SDL_GetError());
        exit(1);
    }


    frame_nr = 0;
    total_sec = 0.0;

    quit = 0;

    while (!quit) {
        while (SDL_PollEvent(&event)) {
            switch (event.type) {
                case SDL_KEYDOWN:
                    switch (event.key.keysym.sym) {
                        case SDLK_ESCAPE:
                            quit = 1;
                            break;
                        case 'd':
                            display = !display;
			    frame_nr=0;
			    total_sec = 0.0;
                            break;
                        case 'f':
    			    printf("%.2f fps\n", frame_nr / total_sec);
			    frame_nr=0;
			    total_sec = 0.0;
                            break;
                        default:
                            break;
                    }
                    break;

                case SDL_QUIT:
                    quit = 1;
                    break;
            }
        }

        if (gettimeofday(&t1, NULL)) {
            perror("gettimeofday");
            exit(1);
        }

        if (read(fd, buf, vwin.width*vwin.height) != (int)(vwin.width*vwin.height)) {
            perror("read");
            exit(1);
        }

	if (display) {

	  BayerNearestNeighbor(buf, buf2, vwin.width, vwin.height, BAYER_DECODING_NEAREST);

          if (SDL_BlitSurface(frame, NULL, screen, NULL)) {
            fprintf(stderr, "SDL_BlitSurface(): %s: FAILED\n", SDL_GetError());
            exit(1);
          }

          if (SDL_Flip(screen)) {
            fprintf(stderr, "SDL_Flip(): %s: FAILED\n", SDL_GetError());
            exit(1);
          }

        }

        frame_nr++;
        if (gettimeofday(&t2, NULL)) {
            perror("gettimeofday");
            exit(1);
        }
        if (timeval_subtract(&et, &t2, &t1)) {
            printf("this should never happen\n");
            exit(1);
        }
        if (et.tv_sec)
            printf("tv_sec != 0\n");
        total_sec += et.tv_usec / 1000000.0;
    }

    close(fd);

    free(buf2);
    free(buf);

    return 0;
}
