/*
 * test1.c
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/videodev.h>

#define BUFSZ   (1280*1024)

static unsigned char buf[BUFSZ];

static void save_pgm(const char *fname, const char *buf, int width, int height)
{
    FILE *fp;

    fp = fopen(fname, "w");
    if (!fp) {
        perror("fopen");
        exit(1);
    }

    fprintf(fp, "P5\n%d %d\n255\n", width, height);

    if (fwrite(buf, width*height, 1, fp) != 1) {
        perror("fwrite");
        exit(1);
    }

    fclose(fp);
}

int main(int argc, char *argv[])
{
    int fd;
    int result;
    struct video_window vwin;

    if (argc != 4) {
        fprintf(stderr, "usage: %s FILENAME WIDTH HEIGHT\n", argv[0]);
        exit(1);
    }

    fd = open("/dev/video", O_RDWR);
    if (fd < 0) {
        perror("open");
        exit(1);
    }

    result = ioctl(fd, VIDIOCGWIN, &vwin);
    if (result < 0) {
        perror("ioctl(VIDIOCGWIN)");
        exit(1);
    }

    vwin.width = atoi(argv[2]);
    vwin.height = atoi(argv[3]);

    result = ioctl(fd, VIDIOCSWIN, &vwin);
    if (result < 0) {
        perror("ioctl(VIDIOCSWIN)");
        exit(1);
    }

    result = read(fd, buf, vwin.width*vwin.height);
    if (result < 0) {
        perror("read");
        exit(1);
    }

    close(fd);

    printf("size=%d\n", vwin.width*vwin.height);
    printf("result=%d\n", result);

    save_pgm(argv[1], buf, vwin.width, vwin.height);
    return 0;
}
